//----------------------------------------------------------------------------
// 
//	file: log.c
// 
//	function: process application logs 
//		- add to an open file or append to a closed file
//		- time stamp every log entry
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Fri Jul 17 04:16:15 CDT 1998
// 
//	$Revision: 1.1 $ $Date: 1998/07/23 01:07:05 $
//	$Log: log.cc,v $
//	Revision 1.1  1998/07/23 01:07:05  yschoe
//	jul22 stable version (except netlayer)
//
//
//----------------------------------------------------------------------------

#include "log.h"

//----------------------------------------------------------------------------
//	log() 
//	  - constructor
//----------------------------------------------------------------------------
log::log(char *comment, char *filename) {

  // open file (actually, it's a good idea to append instead)
  fp = fopen(filename,"a");

  // copy file name for future use
  log::filename = new char [strlen(filename)+1];
  strncpy(log::filename,filename,strlen(filename)+1);

  // set filestat
  filestat = LOG_OPEN;

  // log comment
  log::add("-------------------------------------------------------");
  log::add("BEGIN LOG {%s}",comment);
}

//----------------------------------------------------------------------------
//	~log() 
//	  - destructor
//----------------------------------------------------------------------------
log::~log() {

  // timestamp and end log
  if (filestat == LOG_OPEN) {
	add("%s","END LOG");
	fclose(fp);
  } else if (filestat == LOG_CLOSED) {
	append("%s","END LOG");
  }

  delete [] filename;

}


//----------------------------------------------------------------------------
//	add() 
//	  - add log entry to an open file -- append if it is closed
//----------------------------------------------------------------------------
void log::add(const char *format, ...) {

  va_list vlist;

  // check if the file is already open
  if (filestat == LOG_OPEN) {
	va_start(vlist,format);
        timestamp();
	vfprintf(fp,format,vlist);
	fprintf(fp,"\n");
	va_end(vlist);
  } else if (filestat == LOG_CLOSED) { // keep it closed if it was closed
	fp=fopen(filename,"a"); filestat = LOG_OPEN;
	va_start(vlist,format);
        timestamp();
	vfprintf(fp,format,vlist);
	fprintf(fp,"\n");
	va_end(vlist);
	fclose(fp); filestat = LOG_CLOSED;
  }
}


//----------------------------------------------------------------------------
//	append() 
//   	- open, append, and close log file
//	- once append() was called, both add() and append() acts like
//	append().
//----------------------------------------------------------------------------
void log::append(const char *format, ...) {

  va_list vlist;

  // check if the file is already open
  if (filestat == LOG_OPEN) { // close it even if it was open
        va_start(vlist,format);
        timestamp();
        vfprintf(fp,format,vlist); 
	fprintf(fp,"\n");
        va_end(vlist);
	fclose(fp); filestat = LOG_CLOSED;
  } else if (filestat == LOG_CLOSED) { 
        fp=fopen(filename,"a"); filestat = LOG_OPEN;
        va_start(vlist,format);
        timestamp();
        vfprintf(fp,format,vlist);
	fprintf(fp,"\n");
        va_end(vlist);
        fclose(fp); filestat = LOG_CLOSED;
  }

}


//----------------------------------------------------------------------------
//	timestamp() 
//	  - log the time and date : assume *fp is open
//----------------------------------------------------------------------------
void log::timestamp() {

  time_t t;
  struct tm *local;
  char *month[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
	"Aug", "Sep", "Oct", "Nov", "Dec" };

  // get current time
  time(&t);
  local = localtime(&t); 

  fprintf(fp,"%s %d %02d:%02d:%02d : ",month[local->tm_mon],local->tm_mday,
		local->tm_hour, local->tm_min, local->tm_sec);

}
