//----------------------------------------------------------------------------
// 
//	file: outstarlayer.cc
// 
//	function: make lat weights outstar for better performance 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Tue Aug 18 12:10:38 CDT 1998
// 
//	$Revision: 1.1 $ $Date: 1998/09/04 03:36:37 $
//	$Log: outstarlayer.cc,v $
//	Revision 1.1  1998/09/04 03:36:37  yschoe
//	fixed all warnings generated by -Wall
//
//
//----------------------------------------------------------------------------

#include "outstarlayer.h"

//----------------------------------------------------------------------------
//	outstarlayer() 
//	  - Constructor
//	  - Only deal with stuff that is directly related to the 
//	  	network topology. Drop all stuff such as threshold, etc.
//----------------------------------------------------------------------------
outstarlayer::outstarlayer(char *name, int cols, int rows, 
		 int exc_rad=0, int inh_rad=0,
                 int max_inp_layers=0, netlayer *feeding = (outstar *)NULL,
		 int inp_cols=0, int inp_rows=0, int  aff_rad=0)
: netlayer(name, cols, rows, exc_rad, inh_rad, max_inp, feeding, 
	inp_cols, inp_rows, aff_rad) {

  // do nothing here

}

//----------------------------------------------------------------------------
//	~outstarlayer() 
//	  - descructor
//----------------------------------------------------------------------------
outstarlayer::~outstarlayer() {

  // do nothing here
} 

//----------------------------------------------------------------------------
//	calc_netinput() 
//	  - calculate netinput
//----------------------------------------------------------------------------
void outstarlayer::calc_netinput() {

  REAL	aff_netinp, exc_netinp, inh_netinp;
  int	basex, basey;

  // 
  // 0. clear out net_input for this iteration
  //
  for (int i=0; i<cols; ++i) {
     for (int j=0; j<rows; ++j) {
	net_input[i][j]=0.0; net_exc_input[i][j]=0.0; net_inh_input[i][j]=0.0;
     }
  }

  //
  // 1. Calculate netinput
  //
  for (int i=0; i<cols; ++i) {
     for (int j=0; j<rows; ++j) {

  // 1.1 For all afferent weights of unit [i][j]

        for (int k=0; k<num_inp_layers; ++k) {
           aff_netinp = 0.0; 
  	   basex = rf_ranges[k][i][j].lowx;
	   basey = rf_ranges[k][i][j].lowy;
	   for (int x=rf_ranges[k][i][j].lowx; 
		    x<=rf_ranges[k][i][j].highx; ++x)	
	      for (int y=rf_ranges[k][i][j].lowy; 
		    y<=rf_ranges[k][i][j].highy; ++y)	
	   		aff_netinp += feedings[k]->output[x][y] 
				   * wgt[i][j].aff_ws[k][x-basex][y-basey];

	   net_input[i][j] += aff_contris[k]*aff_netinp;
	}


	// 1.1 done
	
  // 1.2 For all excitatory lateral connections
  // 	later than: v1.12 use outstar calculation here.

	// do outstar only if this unit was active
  	basex = lat_exc_range[i][j].lowx;
	basey = lat_exc_range[i][j].lowy;
	if (output[i][j]>MIN_REAL) {
        for (int x=lat_exc_range[i][j].lowx; 
		 x<=lat_exc_range[i][j].highx; ++x) 
           for (int y=lat_exc_range[i][j].lowy; 	
		    y<=lat_exc_range[i][j].highy; ++y)     
		net_exc_input[x][y] += output[i][j] 
				* wgt[i][j].lat_exc_w[x-basex][y-basey];
	}
	// 1.2 done

  // 1.3 For all inhibitory lateral connections
  // 	later than: v1.12 use outstar calculation here.

	// do outstar only if this unit was active
  	basex = lat_inh_range[i][j].lowx;
	basey = lat_inh_range[i][j].lowy;
	if (output[i][j]>MIN_REAL) {
        for (int x=lat_inh_range[i][j].lowx; 
		 x<=lat_inh_range[i][j].highx; ++x) 
           for (int y=lat_inh_range[i][j].lowy; 
	 	 y<=lat_inh_range[i][j].highy; ++y)     
		net_inh_input[x][y] += output[i][j] 
				* wgt[i][j].lat_inh_w[x-basex][y-basey];
	}
	// 1.3 done
     }

  } // 1. done : net_input[i][j] has the net input now.

  // calculate the final net_input
  for (int i=0; i<cols; ++i) {
     for (int j=0; j<rows; ++j) {
	net_input[i][j] += lat_exc_contri * net_exc_input[i][j] 
			 - lat_inh_contri * net_inh_input[i][j];
     }
  }

}

//----------------------------------------------------------------------------
//	learn() 
//	  - using delta(), modify weights.
//----------------------------------------------------------------------------
void netlayer::learn() {

  int basex, basey;

  for (int i=0; i<cols; ++i) {
     for (int j=0; j<rows; ++j) {

  // 1.1 For all afferent weights of unit [i][j]

        for (int k=0; k<num_inp_layers; ++k) {
	   basex = rf_ranges[k][i][j].lowx;
	   basey = rf_ranges[k][i][j].lowy;
	   for (int x=0; x<rf_ranges[k][i][j].cols; ++x)	
	      for (int y=0; y<rf_ranges[k][i][j].rows; ++y)	
	         wgt[i][j].aff_ws[k][x][y] += delta(i,j,k,x+basex,y+basey);
	}
	
	// 1.1 done
	
  // 1.2 For all excitatory lateral connections

	// outstar
	if (output[i][j]>MIN_REAL) {

	basex = lat_exc_range[i][j].lowx;
	basey = lat_exc_range[i][j].lowy;
        for (int x=0; x<lat_exc_range[i][j].cols; ++x) 
           for (int y=0; y<lat_exc_range[i][j].rows; ++y)
	      wgt[i][j].lat_exc_w[x][y] += delta(i,j,x+basex,y+basey,
						 alpha_lat_exc);

	}
        // 1.2 done

  // 1.3 For all inhibitory lateral connections

	// outstar
	if (output[i][j]>MIN_REAL) {

	basex = lat_inh_range[i][j].lowx;
	basey = lat_inh_range[i][j].lowy;
        for (int x=0; x<lat_inh_range[i][j].cols; ++x) 
           for (int y=0; y<lat_inh_range[i][j].rows; ++y)     
	      wgt[i][j].lat_inh_w[x][y] += delta(i,j,x+basex,y+basey,
						alpha_lat_inh);
	}
	// 1.3 done
     }

  } // 1. done : all weights were modified -- it has yet to be normalized.

}


//----------------------------------------------------------------------------
//	delta() 
//	  - calculate delta, on which weight adaptation is based on
//----------------------------------------------------------------------------
inline REAL netlayer::delta(int col, int row, int cur, int x, int y) {

// for afferent weights : the current input layer is specified by 'cur_inp'

  return (alpha_affs[cur] * (output[col][row] * feedings[cur]->output[x][y]));

}
//----------------------------------------------------------------------------
inline REAL netlayer::delta(int col, int row, int x, int y, REAL alpha) {

// for lateral weights

  return (alpha * (output[col][row]*output[x][y]));

}

//----------------------------------------------------------------------------
//	get_parameter() 
//	  - get parameter value : always return in REAL. Only supports
//	    scalar variables. Returns 0.0 in all other cases.
//----------------------------------------------------------------------------
REAL netlayer::get_parameter(char *param) {

  char param_name[BB_NAME_LEN]; 
  int  cols, rows;
  int param_idx;

  parser *cmdp = new parser(param);

  cmdp->match_param(param_name,&cols,&rows);

  param_idx = net_bb->find(param_name);

  switch (net_bb->table[param_idx].type) {
      case BB_INT: return (REAL)(*(net_bb->table[param_idx].data.i));
      case BB_IP: return (REAL)(net_bb->table[param_idx].data.ip[cols]);
      case BB_IPP: return (REAL)(net_bb->table[param_idx].data.ipp[cols][rows]);
      case BB_REAL: return *(net_bb->table[param_idx].data.r);
      case BB_RP: return net_bb->table[param_idx].data.rp[cols];
      case BB_RPP: return net_bb->table[param_idx].data.rpp[cols][rows];
  }

  delete cmdp;

}
//----------------------------------------------------------------------------
REAL netlayer::get_parameter(char *param, int col) {
	
  char param_name[BB_NAME_LEN];

  sprintf(param_name,"%s[%d]",param,col);
  
  return get_parameter(param_name);

}
//----------------------------------------------------------------------------
REAL netlayer::get_parameter(char *param, int col, int row) {
	
  char param_name[BB_NAME_LEN];

  sprintf(param_name,"%s[%d][%d]",param,col,row);
  
  return get_parameter(param_name);

}


//----------------------------------------------------------------------------
//	reset_matrix() 
//	  - reset matrix to all zeros
//----------------------------------------------------------------------------
void netlayer::reset_matrix(int **mat, int cols, int rows) {

  for (int x=0; x<cols; ++x)
    for (int y=0; y<cols; ++y)
	mat[x][y] = 0;
    
}

//----------------------------------------------------------------------------
void netlayer::reset_matrix(REAL **mat, int cols, int rows) {

  for (int x=0; x<cols; ++x)
    for (int y=0; y<cols; ++y)
	mat[x][y] = 0.0;

}


//----------------------------------------------------------------------------
//	reset_output() 
//	  - reset output activity
//----------------------------------------------------------------------------
void netlayer::reset_output() {

   reset_matrix(output,cols,rows);

}


//----------------------------------------------------------------------------
//	resize_lat() 
//	  - resize lateral exc/inh range
//----------------------------------------------------------------------------
void netlayer::resize_lat_exc_rad(int new_rad) {

  // 1. calculate range and allocate new, while copying over
  //	old weights

  NET_RF_Range **lat_exc_range = alloc_matrix(cols,rows,lat_exc_range);
  REAL **temp_exc_w;

  for (int i=0; i<cols; ++i) {
     for (int j=0; j<rows; ++j) {

	// 1.1 calculate lateral excitatory ranges
 	lat_exc_range[i][j].lowx = MAX(0,i-new_rad);
 	lat_exc_range[i][j].lowy = MAX(0,j-new_rad);
 	lat_exc_range[i][j].highx= MIN(cols-1,i+new_rad);
 	lat_exc_range[i][j].highy= MIN(rows-1,j+new_rad);
	lat_exc_range[i][j].cols = lat_exc_range[i][j].highx 
		- lat_exc_range[i][j].lowx + 1;
	lat_exc_range[i][j].rows = lat_exc_range[i][j].highy
	      	- lat_exc_range[i][j].lowy + 1;
 
	// 1.2 Now allocate memory for the lat_exc weights (in a temp
	//	variable temp_exc_w.
	temp_exc_w = alloc_matrix(lat_exc_range[i][j].cols,
                lat_exc_range[i][j].rows);

    	// 1.3 Clear new one's lat_exc weights
	for (int x=0; x<lat_exc_range[i][j].cols; ++x) 
	   for (int y=0; y<lat_exc_range[i][j].rows; ++y)
		temp_exc_w[x][y] = 0.0;
		
 	// 1.4 Now copy over the old weights to the new wgt matrix
 	int new_offset, old_offset;
 	if (lat_exc_rad > new_rad) {
	  old_offset = lat_exc_rad-new_rad; new_offset = 0;
	} else { 
	  old_offset = 0; new_offset = new_rad-lat_exc_rad;
	}
	for (int x=0; x<lat_exc_range[i][j].cols; ++x) 
	   for (int y=0; y<lat_exc_range[i][j].rows; ++y) 
	   	temp_exc_w[x+new_offset][y+new_offset] 
			= wgt[i][j].lat_exc_w[x+old_offset][y+old_offset];

	// 1.5 free the old weights and link it to the new one
	delete_matrix(netlayer::lat_exc_range[i][j].cols,
		netlayer::lat_exc_range[i][j].rows,wgt[i][j].lat_exc_w);
	wgt[i][j].lat_exc_w = temp_exc_w;

	// 1.7 update the blackboard (no need to free anything here) 
	char tag[BB_NAME_LEN];
	sprintf(tag,"wgts[%d][%d].lat_exc_w",i,j);
	net_bb->update(tag,temp_exc_w,
			lat_exc_range[i][j].cols, lat_exc_range[i][j].rows);
		
     }
  } 

  // 2. Free the old ranges and link it up to the new one
  delete_matrix(cols,rows,netlayer::lat_exc_range);
  netlayer::lat_exc_range = lat_exc_range;

  // 3. update lat_exc_rad
  netlayer::lat_exc_rad = new_rad;

}

//----------------------------------------------------------------------------
//	kill_lat_weights() 
//	  - kill weak connections
//----------------------------------------------------------------------------
void  netlayer::kill_lat_weights(REAL threshold) {

  for (int i=0; i<cols; ++i) {
     for (int j=0; j<rows; ++j) {
	// 1. for all exc weights -- skip this for now
	// 2. for all inh weights
	for (int x=0; x<lat_inh_range[i][j].cols; ++x) {
	   for (int y=0; y<lat_inh_range[i][j].rows; ++y) {
		if (wgt[i][j].lat_inh_w[x][y] <= threshold) {
		   wgt[i][j].lat_inh_w[x][y] = MIN_REAL;
		}
	   }
	}
	// 2. End
     }
  }

}
