//----------------------------------------------------------------------------
// 
//	file: plot_window.h
// 
//	function: plot graph of incoming stream of data 
//		scrolling, auto y-scale adjustment, etc.
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Thu Jul  9 04:38:51 CDT 1998
// 
//	$Revision: 1.3 $ $Date: 2000/02/22 03:54:53 $
//	$Log: plot_window.h,v $
//	Revision 1.3  2000/02/22 03:54:53  yschoe
//	*** empty log message ***
//
//	Revision 1.2  1998/08/11 17:07:10  yschoe
//	1.fixed min/max stuff for more generic use.
//	2. fixed nasty bug in redraw() that prevented some later instances
//	of plot_window not being redrawn.
//
//	Revision 1.1  1998/07/23 01:07:05  yschoe
//	jul22 stable version (except netlayer)
//
//
//----------------------------------------------------------------------------
#ifndef __PLOT_WINDOW_H
#define	__PLOT_WINDOW_H

#include <window.h>
#include <iostream.h>
#include "util.h"
#include "real.h"
#include "matrix_window.h"

//----------------------------------------------------------------------------
//	CLASS: plot_window() 
//	  - displaying graph of datapoints (positive REAL values)
//----------------------------------------------------------------------------
class plot_window : public matrix_window {

protected:
  int	x_origin;	// for cooridinate plotting
  int	y_origin;
  int	x_size;		// for coordinate plotting
  int	y_size;
  int	start_loc;	// window starts here in the data array
  int	cur_loc;	// current location in data array
  int	data_count;	// number of data points in array
  REAL  x_max;		// for calculating relative coordinates
  REAL  x_min;
  REAL  y_max;		// for calculating relative coordinates
  REAL  y_min;		// for calculating relative coordinates
  int	blue,red,green,cyan;	// colors
   
  REAL	x_scale;	// x axis scale when displaying  
  REAL	y_scale;	// y_scale when displaying
  REAL  zoom;		// zoom factor for sliding window
  int	max_points;	// max data points allowed (before cycling)
  int	window_size;	// display window size

  virtual void	init(int max_points);	// init values related to width and height
  virtual void DrawLine(REAL s_x, REAL s_y, REAL d_x, REAL d_y); // draw line
  virtual void CopyArea(REAL s_x, REAL s_y, REAL s_x2, REAL s_y2, 
			REAL d_x, REAL d_y); // draw line

public:

  // use REAL *oned; for data point storage
  // see matrix_window public 

  	plot_window(window &parent, int w, int h, int x, int y, int bw,
		int color_range, int color_mode, 
		int max_points, int window_size);
  	plot_window(window &parent, int w, int h, int x, int y, int bw,
		int color_range, int color_mode, 
		int max_points, REAL *oned);
  	~plot_window();

  virtual void 	draw_interior();	// actual display routine
  virtual void	set_zoom(REAL s);	// set zoom factor (window size)
  virtual REAL  get_zoom();		// return zoom factor
  virtual char  **disp_values();	// display values in a text-popup
  virtual void	resize(int x,int y);	// override resize()
  virtual void	set_scale(REAL x,REAL y); // set x and y axis scale
  virtual void	push_data(REAL value);  // push data into oned[]
  virtual void	push_data(int value);   // push data into oned[]

};
#endif
