
// see backprop.cc






//----------------------------------------------------------------------------
// 
//	file: xor.cc
// 
//	function: test bplayer.cc with an derived class that does xor 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Tue Jul 28 01:46:14 CDT 1998
// 
//	$Revision: 1.2 $ $Date: 2000/01/30 14:55:55 $
//	$Log: xor.cc,v $
//	Revision 1.2  2000/01/30 14:55:55  yschoe
//	*** empty log message ***
//
//	Revision 1.1  1998/10/16 10:35:21  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------

#include <nnview/real.h>
#include <nnview/netlayer.h>
#include <nnview/simulog.h>
#include <nnview/real.h>
#include <nnview/plot_window.h>
#include <nnview/matrix_window.h>
#include <nnview/tool_window.h>

#define xor(x,y) (((x)==(y))?0:1)

//----------------------------------------------------------------------------
//	CLASS: bplayer() 
//	  - back prop layer
//----------------------------------------------------------------------------
enum BP_TYPE {BP_INP, BP_HID, BP_OUT};
 
class bplayer : public netlayer {

public:

  REAL   **d ; // delta matrix
  REAL   **teacher;
  BP_TYPE type;

  virtual inline REAL act_func (REAL netinput);


        bplayer(char *name, int cols, int rows, int max_inp, 
		bplayer *feeding=NULL, int inp_cols=0, int inp_rows=0, 
		int aff_rad=0, BP_TYPE type=BP_INP);
        ~bplayer();

        virtual inline REAL delta(int col, int row, int cur, int inp_col, int inp_row);
	REAL error();

};


plot_window 	*pw ;
matrix_window 	*matw;
matrix_window 	*matw2;
matrix_window 	*matw3;
matrix_window 	*matw4;
matrix_window 	*matw5;
matrix_window 	*matw6;
matrix_window 	*matw7;
matrix_window 	*matw8;
tool_window 	*tw;
int 		steps;
	
simulog *netlog;
bplayer *input;
bplayer *hidden;
bplayer *output;



//----------------------------------------------------------------------------
//	act_func() 
//	  - bplayer::act_func -- activation function
//----------------------------------------------------------------------------
inline REAL bplayer::act_func(REAL netinput) {

  printf ("act_func(%f) = %f\n",netinput,1.0/(1.0+exp(-netinput)));
  return 1.0/(1.0+exp(-netinput));

}

//----------------------------------------------------------------------------
//	bplayer() 
//	  - bplayer::bplayer constructor
//----------------------------------------------------------------------------
bplayer::bplayer(char *name, int cols, int rows, int max_inp,
		 bplayer *feeding=NULL, int inp_cols=0, int inp_rows=0, 			 int  aff_rad=0, BP_TYPE type=BP_INP)
        : netlayer (name, cols, rows, 0, 0, max_inp, feeding,
                    inp_cols, inp_rows, aff_rad) {

  d	  = alloc_matrix(cols,rows);
  teacher = alloc_matrix(cols,rows);
  bplayer::type = type;

  //init_weights(-0.5,0.5);
  init_weights(0.0,0.1);

}

//----------------------------------------------------------------------------
//	~bplayer() 
//	  - bplayer::~bplayer destructor
//----------------------------------------------------------------------------
bplayer::~bplayer() {

  delete_matrix(cols,rows,d);
  delete_matrix(cols,rows,teacher);

}

//----------------------------------------------------------------------------
//	delta() 
//	  - calculate deltaWeight 
//----------------------------------------------------------------------------
REAL bplayer::delta (int col, int row, int cur, int inp_col, int inp_row) {

  REAL alpha = alpha_affs[cur];
  REAL DELTAcur;
  REAL Gprime;

  if (cur!=0) return 0.0; // do this for the first incoming layer

  // col and inp_col is always 0.
  if (type==BP_OUT) {
	REAL Xi     = teacher[col][row];
	REAL V      = output[col][row];
	Gprime = V * (1-V);
	DELTAcur    = Gprime * (Xi-V);
	printf("  delta: %f*(%f-%f)=%f G*(Xi-V)\n",
		Gprime,Xi,V,DELTAcur);
  } else {
	REAL SUMwd = 0.0;
	for (int i=0; i<feedings[1]->rows; ++i) {
		int basex = feedings[1]->rf_ranges[0][0][i].lowx;
		int basey = feedings[1]->rf_ranges[0][0][i].lowy;
		REAL w = feedings[1]->wgt[0][i].aff_ws[0][col-basex][row-basey];
		REAL DELTAprev = ((bplayer *)feedings[1])->d[0][i];
		printf(" %f*%f w * DELTAprev \n",w,DELTAprev);
		SUMwd += w * DELTAprev;
		
        }
	REAL V = output[col][row];
	Gprime = V * (1-V);
	DELTAcur = Gprime * SUMwd;
	printf("  delta: %f*%f=%f G*SUM(w*deltaprev)\n",
		Gprime, SUMwd, DELTAcur);
  }
  d[col][row] =  DELTAcur;

  REAL Vprev = feedings[0]->output[inp_col][inp_row];
  REAL ret = alpha * DELTAcur * Vprev;

  printf("W[%d][%d] Vprv%d=%f G'%d=%f " 
	 "DELTcur=%f deltW=%f\n", row,inp_row,inp_row,Vprev,
	 row,Gprime, DELTAcur, ret);

  return ret;

}

//----------------------------------------------------------------------------
//	error() 
//	  - bplayer : output error sum
//----------------------------------------------------------------------------
REAL bplayer::error() {

  REAL err = 0.0;

  for (int i=0; i<cols; ++i)
	err += (teacher[0][i]-output[0][i])*(teacher[0][i]-output[0][i]);

  return err/2.0;
}


//----------------------------------------------------------------------------
//	setup() 
//	  - bplayer : setup network for simulation
//----------------------------------------------------------------------------
void setup() {

  netlog = new simulog("xor.cc : simple backprop","xor.log");

  // input layer
  cout << "Initializing input layer.\n";
  input = new bplayer("input",1,2,0,(bplayer *)NULL,0,0,0,BP_INP);

  // hidden layer
  cout << "Initializing hidden layer.\n";
  hidden = new bplayer("hidden",1,4,2,input,1,2,2,BP_HID);

  // output layer
  cout << "Initializing output layer.\n";
  output = new bplayer("output",1,2,1,hidden,1,4,10,BP_OUT);

  // add output layer to hidden layer's input for backprop
  hidden->add_inp_layer(output,1,2,0);

  // set parameter values
  hidden->aff_contris[0]=1.0;
  hidden->lat_exc_contri= 0.0;
  hidden->lat_inh_contri= 0.0;
  hidden->aff_noises[0]= 0.0;
  hidden->intrinsic_noise=0.0;
  hidden->alpha_affs[0]=0.01;
  cout << "hidden alpha " << hidden->alpha_affs[0] << endl;
  
  // set parameter values
  output->aff_contris[0]=1.0;
  output->lat_exc_contri= 0.0;
  output->lat_inh_contri= 0.0;
  output->aff_noises[0]= 0.0;
  output->intrinsic_noise=0.0;
  output->alpha_affs[0]=0.01;
  cout << "output alpha " << output->alpha_affs[0] << endl;

}


//----------------------------------------------------------------------------
//	run() 
//	  - bplayer : run a certain number of backprop learning
//----------------------------------------------------------------------------
void run () {

  static count=0;
  int iter;

  for (int i=0; i<steps; ++i) {

     // cout << "iter[" << i << "] : activate and learn.\n";

     cout << "[count=" << count << "]--------------------------------" << endl;
     // set input
     input->output[0][0] = (REAL)((count%4)%2);
     input->output[0][1] = (REAL)(count%4/2);
     // set teacher output
     output->teacher[0][0] = 0.0;
     output->teacher[0][1] = 0.0;

     //if (xor((count%4)%2,count%4/2)) { // xor
     if (((count%4)%2)==1 && (count%4/2)==1) { 
     	output->teacher[0][0] = 0.0;
     	output->teacher[0][1] = 1.0;
	iter = 1;
     } else {
     	output->teacher[0][0] = 1.0;
     	output->teacher[0][1] = 0.0;
	iter = 1;
     }
     count++;
     
     for (int k=0; k<iter; ++k) {
     hidden->activate();
     output->activate();

     output->learn();
     hidden->learn();
     }

     printf("	input= %1d %1d : net_input = %f %f answer = %d %d\n",
		int(input->output[0][0]), int(input->output[0][1]),
		output->net_input[0][0], output->net_input[0][1],
		// output->output[0][0], output->output[0][1],
		int(output->teacher[0][0]), int(output->teacher[0][1]));
     printf("   hidden : net_input = %f %f %f %f \n",
		hidden->net_input[0][0],
		hidden->net_input[0][1],
		hidden->net_input[0][2],
		hidden->net_input[0][3]);
     printf("   error = %f\n",output->error());

    matw->redraw();
    matw2->redraw();
    matw3->redraw();
    matw4->redraw();
    matw5->redraw();
    matw6->redraw();
    matw7->redraw();
    matw8->redraw();
    pw->push_data(output->error());
  }
}

//----------------------------------------------------------------------------
//	set_scale() , set_scale2()
//	  - scroll bar call backs
//----------------------------------------------------------------------------
scrollbar *scbr;
void set_scale() {
  matw->set_scale(scbr->value);
  matw2->set_scale(scbr->value);
  matw3->set_scale(scbr->value);
  matw4->set_scale(scbr->value);
  matw5->set_scale(scbr->value);
  matw6->set_scale(scbr->value);
  matw7->set_scale(scbr->value);
  matw8->set_scale(scbr->value);
}
  
scrollbar *scbr2;
void set_scale2() {
   steps = (int)scbr2->value;
   cout << "steps = " << steps << endl;
}
  
//----------------------------------------------------------------------------
//
//	main() 
//	  - MAIN
//
//----------------------------------------------------------------------------
int main (int argc, char *argv[]) {
  
  main_window *mw = new main_window(argv[0],600,200);

  pw = new plot_window(*mw,600,150,0,0,0,10,COLOR,200,200);
  
  tw = new tool_window("weight","weight",300,300,0,0);


  callback_button *runit = new callback_button(*mw,"Run",run,
                mw->width/4,20,mw->width/2,mw->height-20);
  new quit_button(*mw,mw->width/4,20,mw->width*3/4,mw->height-20);

  setup();

  matw = new matrix_window(*tw,10,300,0,0,0,50,COLOR,1,2,
	output->output[0] );
  matw2 = new matrix_window(*tw,10,300,11,0,0,50,COLOR,1,2,
	output->teacher[0] );
  matw3 = new matrix_window(*tw,10,300,22,0,0,50,COLOR,1,4,
	output->wgt[0][0].aff_ws[0][0] );
  matw4 = new matrix_window(*tw,10,300,33,0,0,50,COLOR,1,4,
	output->wgt[0][1].aff_ws[0][0] );
  matw5 = new matrix_window(*tw,10,300,44,0,0,50,COLOR,1,2,
	hidden->wgt[0][0].aff_ws[0][0] );
  matw6 = new matrix_window(*tw,10,300,55,0,0,50,COLOR,1,2,
	hidden->wgt[0][1].aff_ws[0][0] );
  matw7 = new matrix_window(*tw,10,300,66,0,0,50,COLOR,1,2,
	hidden->wgt[0][2].aff_ws[0][0] );
  matw8 = new matrix_window(*tw,10,300,77,0,0,50,COLOR,1,2,
	hidden->wgt[0][3].aff_ws[0][0] );
  scbr = new scrollbar(*mw, &set_scale,mw->width/2,20,0,mw->height-20,
                0.001,10.0,0.1,"scale=%.1lf");
  scbr2 = new scrollbar(*mw, &set_scale2,mw->width/2,20,0,mw->height-40,
                1,200,1,"Step=%3.0lf");
  steps = 1;
  tw->toggle();

  mw->main_loop();

  delete netlog;
}
